@@ -17,13 +17,14 @@ from nerblackbox.modules.ner_training.data_preprocessing.tools.utils import (
     InputExamples,
 )
 from nerblackbox.modules.ner_training.annotation_tags.annotation import Annotation
+from nerblackbox.modules.ner_training.logging.default_logger import DefaultLogger
 from nerblackbox.tests.utils import PseudoDefaultLogger
 from nerblackbox.modules.utils.util_functions import get_dataset_path
 from torch.utils.data import DataLoader, Sampler, RandomSampler, SequentialSampler
 from pkg_resources import resource_filename
 from copy import deepcopy
 from os.path import join, isfile
-from typing import List, Dict, Tuple, Optional, Any
+from typing import List, Dict, Tuple, Optional, Any, Union
 
 PHASES = ["train", "val", "test"]
 SENTENCES_ROWS_UNPRETOKENIZED = List[Dict[str, Any]]
@@ -34,7 +35,7 @@ class DataPreprocessor:
         self,
         tokenizer,
         do_lower_case: bool,
-        default_logger: PseudoDefaultLogger,
+        default_logger: Union[DefaultLogger, PseudoDefaultLogger],
         max_seq_length: int = 64,
     ):
         """
@@ -110,42 +111,70 @@ class DataPreprocessor:
         return input_examples, annotation
 
     def get_input_examples_predict(
-        self, examples: List[str]
-    ) -> Dict[str, InputExamples]:
+        self,
+        examples: List[str],
+        is_pretokenized: bool,
+    ) -> Tuple[
+        Dict[str, InputExamples], List[str], Optional[List[List[Tuple[int, int]]]]
+    ]:
         """
         - get input examples for PREDICT from input argument examples
 
         Args:
             examples:       [list] of [str]
+            is_pretokenized: True if examples are pretokenized
 
         Returns:
-            input_examples: [dict] w/ key = 'predict' & value = [list] of [InputExample]
+            input_examples:         [dict] w/ key = 'predict' & value = [list] of [InputExample]
+            examples_pretokenized:  [list] of [str]
+            pretokenization_offsets: [list] of [list] of [tuples]
         """
         # create input_examples
         if self.do_lower_case:
             examples = [example.lower() for example in examples]
 
+        if is_pretokenized:
+            _data_pretokenized = [
+                {
+                    "text": example,
+                    "tags": " ".join(
+                        ["O" for _ in range(len(example.split()))]
+                    ),  # pseudo tags
+                }
+                for example in examples
+            ]
+            pretokenization_offsets = None
+        else:
+            _data = [
+                {
+                    "text": example,
+                    "tags": [],
+                }
+                for example in examples
+            ]
+            _data_pretokenized, pretokenization_offsets = self._pretokenize_data(_data)
+
+        examples_pretokenized = [elem["text"] for elem in _data_pretokenized]
+
         input_examples = {
             "predict": [
                 InputExample(
                     guid="",
-                    text=example,
-                    tags=" ".join(
-                        ["O" for _ in range(len(example.split()))]
-                    ),  # pseudo tags
+                    text=elem["text"],
+                    tags=elem["tags"],
                 )
-                for example in examples
+                for elem in _data_pretokenized
             ]
         }
 
-        return input_examples
+        return input_examples, examples_pretokenized, pretokenization_offsets
 
     def to_dataloader(
         self,
         input_examples: Dict[str, InputExamples],
         annotation_classes: List[str],
         batch_size: int,
-    ) -> Dict[str, DataLoader]:
+    ) -> Tuple[Dict[str, DataLoader], Dict[str, List[int]]]:
         """
         - turn input_examples into dataloader
 
@@ -158,6 +187,12 @@ class DataPreprocessor:
         Returns:
             _dataloader:    [dict] w/ keys = ['train', 'val', 'test'] or ['predict'] &
                                       values = [torch Dataloader]
+            _offsets:        [dict] w/ keys = ['train', 'val', 'test'] or ['predict'] &
+                                       values = [List] of [int]
+                                       which gives information on how the input_examples where slices,
+                                       e.g. offsets = [0, 2, 3]
+                                       means that the first two slices belong to the first input example (0->2),
+                                       while the third slice belongs to the second input example (2->3).
         """
         # input_example_to_tensors
         input_examples_to_tensors = InputExamplesToTensors(
@@ -168,13 +203,15 @@ class DataPreprocessor:
         )
 
         _dataloader = dict()
+        _offsets = dict()
         for phase in input_examples.keys():
             self.default_logger.log_info(
                 f"[before preprocessing] {phase.ljust(5)} data: {len(input_examples[phase])} examples"
             )
-            encodings = input_examples_to_tensors(
+            encodings, _offsets_phase = input_examples_to_tensors(
                 input_examples[phase], predict=phase == "predict"
             )
+            _offsets[phase] = _offsets_phase
 
             # dataloader
             data = EncodingsDataset(
@@ -207,7 +244,7 @@ class DataPreprocessor:
                 # see https://pytorch-lightning.readthedocs.io/en/stable/benchmarking/performance.html
             )
 
-        return _dataloader
+        return _dataloader, _offsets
 
     ####################################################################################################################
     # HELPER
@@ -232,7 +269,7 @@ class DataPreprocessor:
         else:
             raise Exception(
                 f"ERROR! Did not find train.csv OR train.jsonl in {dataset_path}:"
-                f"(csv: {isfile(dataset_path_csv)}, jsonl: {isfile(dataset_path_jsonl)}"
+                f"(csv: {isfile(dataset_path_csv)}, jsonl: {isfile(dataset_path_jsonl)})"
             )
 
     @staticmethod
@@ -254,9 +291,34 @@ class DataPreprocessor:
                 i += 1
         return _tokens
 
+    @staticmethod
+    def _resolve_overlapping_tags(_tags: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
+        """
+
+        Args:
+            _tags: e.g.
+            [
+             {"token": "Bajo peso", "tag": "Concept", "char_start": 4651, "char_end": 4660},
+             {"token": "peso", "tag": "Concept", "char_start": 4656, "char_end": 4660},
+             {"token": "más", "tag": "Predicate", "char_start": 4681, "char_end": 4684},
+            ]
+
+        Returns:
+            _resolved_tags: e.g.
+            [
+             {"token": "Bajo peso", "tag": "Concept", "char_start": 4651, "char_end": 4660},
+             {"token": "más", "tag": "Predicate", "char_start": 4681, "char_end": 4684},
+            ]
+        """
+        _resolved_tags = list()
+        for i in range(len(_tags)):
+            if i == 0 or _tags[i - 1]["char_end"] <= _tags[i]["char_start"]:
+                _resolved_tags.append(_tags[i])
+        return _resolved_tags
+
     def _pretokenize_data(
         self, _data: SENTENCES_ROWS_UNPRETOKENIZED
-    ) -> List[Dict[str, str]]:
+    ) -> Tuple[List[Dict[str, str]], List[List[Tuple[int, int]]]]:
         """
         pretokenize data (text and text simultaneously)
 
@@ -280,26 +342,41 @@ class DataPreprocessor:
                 },
                 ..
             ]
+            _pretokenization_offsets: e.g. [
+                [(0,17), (18,20), (20,21), (21,26)]
+            ]
         """
         _data_pretokenized = list()
+        _pretokenization_offsets = list()
         for n in range(len(_data)):
-            words = self._tokens2words(self.tokenizer.tokenize(_data[n]["text"]))
-            index = 0
-            tags = ["O"] * len(words)
-            for entity_dict in _data[n]["tags"]:
-                entity_text = entity_dict["token"]
-                entity_words = self._tokens2words(self.tokenizer.tokenize(entity_text))
-                entity_words_indices = [
-                    index + words[index:].index(entity_word)
-                    for entity_word in entity_words
-                ]
-                for i, entity_word_index in enumerate(entity_words_indices):
-                    tags[entity_word_index] = (
-                        f"B-{entity_dict['tag']}"
-                        if i == 0
-                        else f"I-{entity_dict['tag']}"
-                    )
-                index = max(entity_words_indices)
+            word_tuples = (
+                self.tokenizer.backend_tokenizer.pre_tokenizer.pre_tokenize_str(
+                    _data[n]["text"]
+                )
+            )
+            tags = ["O"] * len(word_tuples)
+            entity_dicts = self._resolve_overlapping_tags(_data[n]["tags"])
+            for entity_dict in entity_dicts:
+                entity_char_start = entity_dict["char_start"]
+                entity_char_end = entity_dict["char_end"]
+                entity_tag = entity_dict["tag"]
+                for word_index, word_tuple in enumerate(word_tuples):
+                    word_char_start = word_tuple[1][0]
+                    word_char_end = word_tuple[1][1]
+                    if (
+                        word_char_start == entity_char_start
+                        and word_char_end <= entity_char_end
+                    ):
+                        tags[word_index] = f"B-{entity_tag}"
+                    elif (
+                        word_char_start >= entity_char_start
+                        and word_char_end <= entity_char_end
+                    ):
+                        tags[word_index] = f"I-{entity_tag}"
+
+            words = [word_tuple[0] for word_tuple in word_tuples]
+            pretokenization_offsets = [word_tuple[1] for word_tuple in word_tuples]
+            _pretokenization_offsets.append(pretokenization_offsets)
 
             _data_pretokenized.append(
                 {
@@ -308,7 +385,7 @@ class DataPreprocessor:
                 }
             )
 
-        return _data_pretokenized
+        return _data_pretokenized, _pretokenization_offsets
 
     def _pretokenize(self, dataset_path: str) -> None:  # pragma: no cover
         """
@@ -329,7 +406,7 @@ class DataPreprocessor:
                 data = [json.loads(jline) for jline in jlines]
 
             # 2. pretokenize data
-            data_pretokenized = self._pretokenize_data(data)
+            data_pretokenized, _ = self._pretokenize_data(data)
 
             # 3. write csv files "pretokenized_{phase}.csv"
             df = pd.DataFrame(data_pretokenized)
