@@ -22,6 +22,9 @@ from nerblackbox.modules.ner_training.data_preprocessing.tools.utils import (
     EncodingsKeys,
 )
 from nerblackbox.tests.utils import PseudoDefaultLogger
+from nerblackbox.modules.ner_training.data_preprocessing.tools.utils import (
+    InputExamples,
+)
 
 SENTENCES_ROWS_UNPRETOKENIZED = List[Dict[str, Any]]
 
@@ -32,6 +35,13 @@ tokenizer = AutoTokenizer.from_pretrained(
     use_fast=True,
 )
 
+tokenizer_KB = AutoTokenizer.from_pretrained(
+    "KB/bert-base-swedish-cased",
+    do_lower_case=False,
+    additional_special_tokens=["[newline]", "[NEWLINE]"],
+    use_fast=True,
+)
+
 csv_reader = CsvReader(
     path=resource_filename("nerblackbox", "tests/test_data"),
     tokenizer=tokenizer,
@@ -42,7 +52,7 @@ csv_reader = CsvReader(
 )
 
 data_preprocessor = DataPreprocessor(
-    tokenizer=tokenizer,
+    tokenizer=tokenizer_KB,
     do_lower_case=False,
     default_logger=PseudoDefaultLogger(),
     max_seq_length=4,
@@ -108,7 +118,7 @@ class TestCsvReaderAndDataProcessor:
         self,
         annotation_classes: List[str],
         annotation_scheme: str,
-        input_examples: Dict[str, List[InputExample]],
+        input_examples: Dict[str, InputExamples],
     ) -> None:
 
         ##################################
@@ -158,9 +168,15 @@ class TestCsvReaderAndDataProcessor:
         test_sentences = [
             elem.text for v in test_input_examples.values() for elem in v
         ]  # retrieve example sentences
-        test_input_examples_predict = data_preprocessor.get_input_examples_predict(
-            test_sentences
-        )["predict"]
+        (
+            _test_input_examples_predict,
+            _,
+            _,
+        ) = data_preprocessor.get_input_examples_predict(
+            test_sentences,
+            is_pretokenized=True,
+        )
+        test_input_examples_predict = _test_input_examples_predict["predict"]
         assert len(test_input_examples_predict) == len(
             input_examples["predict"]
         ), f"len(test_input_examples_predict) = {len(test_input_examples_predict)} != {len(input_examples['predict'])}"
@@ -178,7 +194,7 @@ class TestCsvReaderAndDataProcessor:
             ), f"test_input_example_predict.tags = {test_input_example_predict.tags} != {true_input_example_predict_tags}"
 
         # c. to_dataloader
-        dataloader = data_preprocessor.to_dataloader(
+        dataloader, _ = data_preprocessor.to_dataloader(
             input_examples, annotation_classes, batch_size=1
         )
         for key in ["train", "val", "test", "predict"]:
@@ -213,10 +229,60 @@ class TestDataProcessor:
         test_words = data_preprocessor._tokens2words(tokens)
         assert test_words == words, f"test_words = {test_words} != {words}"
 
+    @pytest.mark.parametrize(
+        "tags, resolved_tags",
+        [
+            (
+                [
+                    {
+                        "token": "Bajo peso",
+                        "tag": "Concept",
+                        "char_start": 4651,
+                        "char_end": 4660,
+                    },
+                    {
+                        "token": "peso",
+                        "tag": "Concept",
+                        "char_start": 4656,
+                        "char_end": 4660,
+                    },
+                    {
+                        "token": "más",
+                        "tag": "Predicate",
+                        "char_start": 4681,
+                        "char_end": 4684,
+                    },
+                ],
+                [
+                    {
+                        "token": "Bajo peso",
+                        "tag": "Concept",
+                        "char_start": 4651,
+                        "char_end": 4660,
+                    },
+                    {
+                        "token": "más",
+                        "tag": "Predicate",
+                        "char_start": 4681,
+                        "char_end": 4684,
+                    },
+                ],
+            ),
+        ],
+    )
+    def test_resolve_overlapping_tags(
+        self, tags: List[Dict[str, Any]], resolved_tags: List[Dict[str, Any]]
+    ):
+        test_resolved_tags = data_preprocessor._resolve_overlapping_tags(tags)
+        assert (
+            test_resolved_tags == resolved_tags
+        ), f"test_resolved_tags = {test_resolved_tags} != {resolved_tags}"
+
     @pytest.mark.parametrize(
         "data, data_pretokenized",
         [
             (
+                # data
                 [
                     {
                         "text": "arbetsförmedlingen ai-center finns i stockholm.",
@@ -246,7 +312,48 @@ class TestDataProcessor:
                             },
                         ],
                     },
+                    {
+                        "text": "Enligt Rory har mamman sagt åt honom att barn måste ha tålamod och inte får slå de vuxna. När Rory får veta",
+                        "tags": [
+                            {
+                                "char_start": 7,
+                                "char_end": 11,
+                                "token": "Rory",
+                                "tag": "PI",
+                            },
+                            {
+                                "char_start": 94,
+                                "char_end": 98,
+                                "token": "Rory",
+                                "tag": "PI",
+                            },
+                        ],
+                    },
+                    {
+                        "text": "Socialsekreterare Molly Knös ringer och bokar in möte 2020-08-09.\n2020-08-09",
+                        "tags": [
+                            {
+                                "char_start": 18,
+                                "char_end": 28,
+                                "token": "Molly Knös",
+                                "tag": "PI",
+                            },
+                            {
+                                "char_start": 54,
+                                "char_end": 64,
+                                "token": "2020-08-09",
+                                "tag": "PI",
+                            },
+                            {
+                                "char_start": 66,
+                                "char_end": 76,
+                                "token": "2020-08-09",
+                                "tag": "PI",
+                            },
+                        ],
+                    },
                 ],
+                # data pretokenized
                 [
                     {
                         "text": "arbetsförmedlingen ai - center finns i stockholm .",
@@ -256,6 +363,14 @@ class TestDataProcessor:
                         "text": "this example contains hugging face",
                         "tags": "O O O B-ORG I-ORG",
                     },
+                    {
+                        "text": "Enligt Rory har mamman sagt åt honom att barn måste ha tålamod och inte får slå de vuxna . När Rory får veta",
+                        "tags": "O B-PI O O O O O O O O O O O O O O O O O O B-PI O O",
+                    },
+                    {
+                        "text": "Socialsekreterare Molly Knös ringer och bokar in möte 2020 - 08 - 09 . 2020 - 08 - 09",
+                        "tags": "O B-PI I-PI O O O O O B-PI I-PI I-PI I-PI I-PI O B-PI I-PI I-PI I-PI I-PI",
+                    },
                 ],
             ),
         ],
@@ -265,7 +380,7 @@ class TestDataProcessor:
         data: SENTENCES_ROWS_UNPRETOKENIZED,
         data_pretokenized: List[Dict[str, str]],
     ):
-        test_data_pretokenized = data_preprocessor._pretokenize_data(data)
+        test_data_pretokenized, _ = data_preprocessor._pretokenize_data(data)
         assert (
             test_data_pretokenized == data_pretokenized
         ), f"test_data_pretokenized = {test_data_pretokenized} != {data_pretokenized}"
@@ -517,7 +632,7 @@ class TestInputExamplesToTensorsAndEncodingsDataset:
             annotation_classes_tuple=tuple(annotation_classes),
             default_logger=PseudoDefaultLogger(),
         )
-        encodings = input_examples_to_tensors(input_examples, predict=False)
+        encodings, _ = input_examples_to_tensors(input_examples, predict=False)  # TODO
         input_tokens = [
             tokenizer.convert_ids_to_tokens(input_ids_single)
             for input_ids_single in encodings["input_ids"]
